#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#--------------------------------------------------------------------------------
#www2sms.py v0.2.1
#Copyright Bjoern Olausson
#--------------------------------------------------------------------------------
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#GNU General Public License for more details.
#
#To view the license visit
#http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
#or write to
#Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#--------------------------------------------------------------------------------
#--------------------------------------------------------------------------------
#

import os, errno, pwd, sys, math, sqlite3, time, calendar, evolution, re
from PyQt4 import QtCore, QtGui
from www2sms_gui_main import *
from www2sms_gui_widget_account import *
from www2sms_gui_widget_advanced import *
from www2sms_gui_dialog_progress import Ui_Form_Progress
import www2sms_providers

USER = pwd.getpwuid(os.getuid())[0]
CONFIG_DIR = "/home/%s/MyDocs/www2sms" %(USER)

#Get the list of providers based on the classes implemented.
list_providers = []
providers = []
for label in dir(www2sms_providers):
    if label.startswith('provider_'):
        provider_class = getattr(www2sms_providers, label)
        provider = provider_class()
	list_providers.append(provider.getName())
	providers.append(provider)

class MainWindow(QtGui.QMainWindow, Ui_MainWindow):
	def __init__(self, win_parent = None):
		#Init the base class
		QtGui.QMainWindow.__init__(self, win_parent)
		self.ui = Ui_MainWindow()
		self.ui.setupUi(self)
		# Setup GUI elements
		# Populate the contact list
		self.comboBox_contact_populate()
		# Populate the available Providers
		for value in list_providers:
			AccSet.ui.comboBox_SetupProvider.addItem(value)
		CT = time.localtime(time.time())
		# Set the year for scheduling SMS. From now to the next three years
		YEAR = CT[0]
		for YEARS in range(YEAR,YEAR + 3, 1):
			AdvSet.ui.comboBox_AdvancedScheduleStartYear.addItem(str(YEARS))
			AdvSet.ui.comboBox_AdvancedScheduleStopYear.addItem(str(YEARS))
		# Try to update the GUI to the last used Account, else skip this
		try:
			PROVIDER, ALIAS, LOGIN, PASSWORD = c.execute('SELECT provider, alias, login, password FROM login WHERE last=1').fetchall()[0]
			# Populate all Acounts
			self.ACCOUNTS = c.execute('SELECT alias FROM login').fetchall()
			for a in self.ACCOUNTS:
				AccSet.ui.comboBox_SetupAlias.addItem(a[0])
			# Update the GUI
			# Updating the the Provider ComboBox
			provider_index = AccSet.ui.comboBox_SetupProvider.findText(PROVIDER)
			AccSet.ui.comboBox_SetupProvider.setCurrentIndex(provider_index)
			# Updating the the Alias ComboBox
			alias_index = AccSet.ui.comboBox_SetupAlias.findText(ALIAS)
			AccSet.ui.comboBox_SetupAlias.setCurrentIndex(alias_index)
			# Updating the lineEdit Alias
			AccSet.ui.lineEdit_SetupNewAlias.setText(ALIAS)
			# Updating the lineEdit Login
			AccSet.ui.lineEdit_SetupLogin.setText(LOGIN)
			# Updating the lineEdit Pasword
			#PASSWORD = base64.b64decode(PASSWORD)
			AccSet.ui.lineEdit_SetupPassword.setText(PASSWORD)
		except IndexError:
			provider_index = -1
			self.ACCOUNTS = False

		# Connect to some buttons and make them execute a method of the class MainWindow
		QtCore.QObject.connect(self.ui.pushButton_Send, QtCore.SIGNAL("clicked()"), self.on_send_clicked)
		QtCore.QObject.connect(self.ui.comboBox_contact, QtCore.SIGNAL("activated(QString)"), self.comboBox_contact_itemActivated)
		QtCore.QObject.connect(self.ui.comboBox_number, QtCore.SIGNAL("activated(QString)"), self.comboBox_number_itemActivated)
		QtCore.QObject.connect(AccSet.ui.pushButton_SetupSave, QtCore.SIGNAL("clicked()"), self.on_SetupSave_clicked)
		QtCore.QObject.connect(self.ui.plainTextEdit_SMS, QtCore.SIGNAL("textChanged()"), self.UpdateChars)
		QtCore.QObject.connect(AccSet.ui.comboBox_SetupAlias, QtCore.SIGNAL("activated(QString)"), self.on_SetupAlias_activated)
		QtCore.QObject.connect(AccSet.ui.pushButton_SetupDelete, QtCore.SIGNAL("clicked()"), self.on_SetupDelete_clicked)
		QtCore.QObject.connect(AdvSet.ui.comboBox_AdvancedScheduleStartMonth, QtCore.SIGNAL("activated(QString)"), self.on_StartDate_changed)
		QtCore.QObject.connect(AdvSet.ui.comboBox_AdvancedScheduleStopMonth, QtCore.SIGNAL("activated(QString)"), self.on_StopDate_changed)
		QtCore.QObject.connect(AdvSet.ui.comboBox_AdvancedScheduleStartYear, QtCore.SIGNAL("activated(QString)"), self.on_StartDate_changed)
		QtCore.QObject.connect(AdvSet.ui.comboBox_AdvancedScheduleStopYear, QtCore.SIGNAL("activated(QString)"), self.on_StopDate_changed)
		self.enableTabs(provider_index)
		QtCore.QObject.connect(self.ui.actionAdv_Provider_Settings, QtCore.SIGNAL("triggered(bool)"), self.ShowDialogAdv)
		QtCore.QObject.connect(self.ui.actionAccount_Setup, QtCore.SIGNAL("triggered(bool)"), self.ShowDialogAccSet)
	def enableTabs(self, provider_index):
		## Enable the SMS SEND Button if we have an account, otherwise not
		if self.ACCOUNTS:
			self.ui.pushButton_Send.show()
		else:
			self.ui.pushButton_Send.hide()
		## Enable the "advanced" options if supported
		if (provider_index >= 0) and providers[provider_index].hasAdvancedFeatures():
			self.ui.actionAdv_Provider_Settings.setEnabled(True)
		else:
			self.ui.actionAdv_Provider_Settings.setDisabled(True)

	def ShowDialogAdv(self):
	    	PROVIDER = AccSet.ui.comboBox_SetupProvider.currentText()
	    	# TODO: When there are more providers, a GUI element should be created for every
	    	# TODO: advanced setting. For now only o2online.de has such settings, so AdvSet
	    	# TODO: can be considered the GUI for o2online advanced settings. Thats why we set
	    	# TODO: PROVIDER = 'AdvSet'
	    	PROVIDER = 'AdvSet'
	    	# Dictionary of all available UIs for advanced settings.
		p_dict = {'AdvSet' : AdvSet.show}
		p_dict[PROVIDER]()
		# TODO: Check wy the sheduling for o2online.de is not working.
		# TODO: Meanwhile this is disabled.
		AdvSet.ui.groupBox_AdvancedSchedule.setDisabled(True)

	def ShowDialogAccSet(self):
		AccSet.show()

	# Class wich is called to get the contacts from the evolution backend
	def comboBox_contact_populate(self):
	    	# TODO: Find out why Umlaute like "ä,ö,ü,ß" are garbled.
		# Regular expression to find all phone numbers in the vCard
		re_tel = re.compile('TEL;TYPE')

		name_attributes = [
			"full-name",
			"given-name",
			"family-name",
			"nickname"]

		phone_attributes = [
			"assistant-phone",
			"business-phone",
			"business-phone-2",
			"business-fax",
			"callback-phone",
			"car-phone",
			"company-phone",
			"home-phone",
			"home-phone-2",
			"home-fax",
			"isdn-phone",
			"mobile-phone",
			"other-phone",
			"other-fax",
			"pager",
			"primary-phone",
			"radio",
			"telex",
			"tty",]

		single_contact = []
		single_contact_numbers = []
		self.contact_list = []
		self.phone_list = []

		abook = evolution.ebook.open_addressbook("default")
		contacts = abook.get_all_contacts()
		contacts.sort(key=lambda obj: obj.get_property(name_attributes[2]))

		for econtact in contacts:
			single_contact = [econtact.get_property(name_attributes[2]), econtact.get_property(name_attributes[1]), econtact.get_property(name_attributes[3])]
			vcard = econtact.get_vcard_string()
			for line in vcard.split('\n'):
				if re_tel.search(line):
					single_contact_numbers.append(line.split(':')[1].strip())
			self.contact_list.append(single_contact)
			self.phone_list.append(single_contact_numbers)
			single_contact = []
			single_contact_numbers = []

		for contact in self.contact_list:
			if contact[0] and contact[1]:
				item = "%s, %s" %(contact[0], contact[1])
			elif contact[0] and not contact[1]:
				item = "%s" %(contact[0])
			elif not contact[0] and contact[1]:
				item = "%s" %(contact[1])
			elif not contact[0] and not contact[1] and contact[2]:
				item = "%s" %(contact[2])
			else:
				item = "No name available"
			self.ui.comboBox_contact.addItem(item)

	def comboBox_contact_itemActivated(self):
		self.ui.comboBox_number.clear()
		contact_index = self.ui.comboBox_contact.currentIndex()
		if len(self.phone_list[contact_index]) == 0:
			self.ui.comboBox_number.addItem('No number available')
		else:
			for item in self.phone_list[contact_index]:
				try:
					if not item:
						self.ui.comboBox_number.addItem('No number available')
					else:
						self.ui.comboBox_number.addItem(item)
				except:
					pass
		if len(self.phone_list[contact_index]) == 0:
			self.ui.lineEdit_Number.setText('No number available')
		else:
			self.ui.lineEdit_Number.setText(self.phone_list[contact_index][0])

	def comboBox_number_itemActivated(self):
		item = self.ui.comboBox_number.currentText()
		self.ui.lineEdit_Number.setText(item)

	# Method which is called to get the proper number of days for the schedule start date
	def on_StartDate_changed(self):
		cal = calendar.Calendar()
		sYEAR = int(self.ui.comboBox_AdvancedScheduleStartYear.currentText())
		sMONTH = int(self.ui.comboBox_AdvancedScheduleStartMonth.currentText())
		self.ui.comboBox_AdvancedScheduleStartDay.clear()
		for DAY in cal.itermonthdays(sYEAR, sMONTH):
			if DAY != 00:
				DAY = '%02d' %(DAY)
				self.ui.comboBox_AdvancedScheduleStartDay.addItem(str(DAY))

	# Method which is called to get the proper number of days for the schedule stop date
	def on_StopDate_changed(self):
		cal = calendar.Calendar()
		sYEAR = int(self.ui.comboBox_AdvancedScheduleStopYear.currentText())
		sMONTH = int(self.ui.comboBox_AdvancedScheduleStopMonth.currentText())
		self.ui.comboBox_AdvancedScheduleStopDay.clear()
		for DAY in cal.itermonthdays(sYEAR, sMONTH):
			if DAY != 00:
				DAY = '%02d' %(DAY)
				self.ui.comboBox_AdvancedScheduleStopDay.addItem(str(DAY))

	# Method which is called to get the number of characters typed in plainTextEdit_SMS and update lcdNumber_SMSChars
	def UpdateChars(self):
		SMS_CHARS = self.ui.plainTextEdit_SMS.document().characterCount() - 1
		self.ui.label_SMSChars.setText(str(SMS_CHARS))
		# Might be better. Floor to Ceiling division: -(-x//y)
		SMS_COUNT = int(math.ceil(SMS_CHARS / 160) + 1)
		self.ui.label_SMSCount.setText(str(SMS_COUNT))

	# Method which updates the GUI to the Account selected in comboBox_SetupAlias
	def on_SetupAlias_activated(self):
		# Get the selected Account
		ALIAS = unicode(AccSet.ui.comboBox_SetupAlias.currentText())
		# Read the settings for the selected Account fom DB
		try:
			PROVIDER, ALIAS, LOGIN, PASSWORD = c.execute('SELECT provider, alias, login, password FROM login WHERE alias=?', (ALIAS,)).fetchall()[0]
		except IndexError:
			pass
		# Update the GUI elements
		provider_index = AccSet.ui.comboBox_SetupProvider.findText(PROVIDER)
		AccSet.ui.comboBox_SetupProvider.setCurrentIndex(provider_index)
		AccSet.ui.lineEdit_SetupNewAlias.setText(ALIAS)
		AccSet.ui.lineEdit_SetupLogin.setText(LOGIN)
		AccSet.ui.lineEdit_SetupPassword.setText(PASSWORD)
		self.enableTabs(provider_index)
		# Remove the last account flag to set it on the new/updated profile
		c.execute('UPDATE login SET last=0 WHERE last=1')
		# Set the last flag on the new Account
		c.execute('UPDATE login SET last=1 WHERE alias=?',(ALIAS,))
		conn.commit()

	# Well, this is obvious, isn't it? Okay, it deletes the current selected account.
	def on_SetupDelete_clicked(self):
		# Get the current selected Account
		ALIAS = unicode(AccSet.ui.comboBox_SetupAlias.currentText())
		# Delete it from the DB
		c.execute('DELETE FROM login WHERE alias=?', (ALIAS,))
		conn.commit()
		# Remove the Account from comboBox_SetupAlias
		alias_index = AccSet.ui.comboBox_SetupAlias.findText(ALIAS)
		AccSet.ui.comboBox_SetupAlias.removeItem(alias_index)
		# Check if there is something left in the DB
		ROWS = c.execute('SELECT alias FROM login').rowcount
		# Update the GUI elements
		if ROWS == -1:
			AccSet.ui.lineEdit_SetupNewAlias.clear()
			AccSet.ui.lineEdit_SetupLogin.clear()
			AccSet.ui.lineEdit_SetupPassword.clear()
			self.enableTabs(-1)
		else:
			AccSet.on_SetupAlias_activated()
		QtGui.QMessageBox.information(AccSet, 'www2sms', "Account %s deleted" %(ALIAS))

	# Method which is called, when an account is created/updated
	def on_SetupSave_clicked(self):
		# Gather Account information
		PROVIDER = unicode(AccSet.ui.comboBox_SetupProvider.currentText())
		ALIAS = unicode(AccSet.ui.lineEdit_SetupNewAlias.text())
		LOGIN = unicode(AccSet.ui.lineEdit_SetupLogin.text())
		PASSWORD = unicode(AccSet.ui.lineEdit_SetupPassword.text())
		# Remove the last account flag to set it on the new/updated profile
		c.execute('UPDATE login SET last=0 WHERE last=1')
		# Check if Account exist, if it exists, we update the content in the DB on save, else we add it
		EXIST = c.execute('select alias FROM login WHERE alias=?', (ALIAS,)).fetchall()
		if len(EXIST) != 0:
			c.execute('UPDATE login SET provider=?, login=?, password=?, last=1 WHERE alias=?', (PROVIDER, LOGIN, PASSWORD, ALIAS))
		else:
			c.execute('INSERT INTO login VALUES (NULL, ?, ?, ?, ?, 1)', (PROVIDER, ALIAS, LOGIN, PASSWORD))
			# Add the new Account to comboBox_SetupAlias
		    	AccSet.ui.comboBox_SetupAlias.addItem(ALIAS)
		conn.commit()
		# Let the user know that we have saved the the account/changes
		QtGui.QMessageBox.information(AccSet, 'www2sms', "saved")
		# Update the GUI
		alias_index = AccSet.ui.comboBox_SetupAlias.findText(ALIAS)
		AccSet.ui.comboBox_SetupAlias.setCurrentIndex(alias_index)
		provider_index = AccSet.ui.comboBox_SetupProvider.findText(PROVIDER)
		AccSet.ui.comboBox_SetupProvider.setCurrentIndex(provider_index)
		AccSet.ui.lineEdit_SetupLogin.setText(LOGIN)
		AccSet.ui.lineEdit_SetupPassword.setText(PASSWORD)
		self.enableTabs(provider_index)

	# Method which is called when we want to send the sms. It actually calls a method on the providers class
	def on_send_clicked(self):
		self.ui.pushButton_Send.setText("Sending...")
		# Really ready to send?
		reply = QtGui.QMessageBox.question(self, 'Send SMS?', "Are you sure?", QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
		# If reply is YES, we send the SMS, else we just do nothing
		if reply == QtGui.QMessageBox.Yes:
			self.Stat = FormPro()
		    # Read Scheduling settings if activated
			SCHEDULE = []
			if AdvSet.ui.checkBox_AdvancedScheduleRepeat.isChecked():
				# SCHEDULE Layout: ([RECURRENCY, STOPTrueFalse], [YEARstart, YEARstop], [MONTHstart, MONTHstop], [DAYstart, DAYstop], [HOUREstart, HOUREstop], [MINUTEstart, MINUTEstop])
				SCHEDULE.append([ str(AdvSet.ui.comboBox_AdvancedRecurrence.currentText()), self.ui.checkBox_AdvancedScheduleStop.isChecked() ])
				SCHEDULE.append([ str(AdvSet.ui.comboBox_AdvancedScheduleStartYear.currentText()), str(AdvSet.ui.comboBox_AdvancedScheduleStopYear.currentText()) ])
				SCHEDULE.append([ str(AdvSet.ui.comboBox_AdvancedScheduleStartMonth.currentText()), str(AdvSet.ui.comboBox_AdvancedScheduleStopMonth.currentText()) ])
				SCHEDULE.append([ str(AdvSet.ui.comboBox_AdvancedScheduleStartDay.currentText()), str(AdvSet.ui.comboBox_AdvancedScheduleStopDay.currentText()) ])
				SCHEDULE.append([ str(AdvSet.ui.comboBox_AdvancedScheduleStartHoure.currentText()), str(AdvSet.ui.comboBox_AdvancedScheduleStopHoure.currentText()) ])
				SCHEDULE.append([ str(AdvSet.ui.comboBox_AdvancedScheduleStartMinute.currentText()), str(AdvSet.ui.comboBox_AdvancedScheduleStopMinute.currentText()) ])
			else:
				SCHEDULE = False

			# Read Sender
			if AdvSet.ui.radioButton_AdvancedMyMobileNumber.isChecked():
				SENDER = "MyNumber"
			elif AdvSet.ui.radioButton_AdvancedAnonym.isChecked():
				SENDER = "Anonym"
			elif AdvSet.ui.radioButton_AdvancedText.isChecked():
				SENDER = self.ui.lineEdit_AdvancedText.text()

			# Read Special
			FLASH = AdvSet.ui.checkBox_AdvancedFlashSMS.isChecked()
			# Read the login
			LOGIN = unicode(AccSet.ui.lineEdit_SetupLogin.text())
			# Read the password
			PASSWORD = unicode(AccSet.ui.lineEdit_SetupPassword.text())
			# Read the number to which we send the SMS
			NUMBER = unicode(self.ui.lineEdit_Number.text())
			# Read the text we want to send
			SMSTEXT = unicode(self.ui.plainTextEdit_SMS.toPlainText())
			# Read the provider to use
			PROVIDER = unicode(AccSet.ui.comboBox_SetupProvider.currentText())
			# Get the provder
			provider_index = AccSet.ui.comboBox_SetupProvider.findText(PROVIDER)
			provider = providers[provider_index]
			# send the SMS
			STATUS, MSG = provider.sendSMS(self, app, NUMBER, SMSTEXT, LOGIN, PASSWORD, SCHEDULE, SENDER, FLASH)
			self.Stat.close()
			if STATUS:
				self.on_send_success(MSG)
			else:
				self.on_send_failed(MSG)
		self.ui.pushButton_Send.setText("Send")

	# Method which is called from providers class when the sms was successfully send
	def on_send_success(self,MSG):
		QtGui.QMessageBox.information(self, 'www2sms', MSG)
		self.ui.lineEdit_Number.clear()
		self.ui.plainTextEdit_SMS.clear()

	# Method which is called from providers class when the sms failed to send
	def on_send_failed(self,MSG):
		QtGui.QMessageBox.warning(self, 'www2sms', MSG)

def DialogStatusMessage(self, statusmessage):
	QtGui.QMessageBox.information(self, 'www2sms', statusmessage)

class FormAccount(QtGui.QDialog, Ui_FormAccount):
	def __init__(self, win_parent = None):
		#Init the base class
		QtGui.QDialog.__init__(self, win_parent)
		self.ui = Ui_FormAccount()
		self.ui.setupUi(self)

class FormAdv(QtGui.QDialog, Ui_FormAdvanced):
	def __init__(self, win_parent = None):
		#Init the base class
		QtGui.QDialog.__init__(self, win_parent)
		self.ui = Ui_FormAdvanced()
		self.ui.setupUi(self)

class FormPro(QtGui.QDialog, Ui_Form_Progress):
	def __init__(self, win_parent = None):
		#Init the base class
		QtGui.QDialog.__init__(self, win_parent)
		self.ui = Ui_Form_Progress()
		self.ui.setupUi(self)

def mk_db():
    try:
	os.makedirs(CONFIG_DIR, 0755)
    except OSError, e:
	# Pass if the directory exist, else raise a exception
	if e.errno == errno.EEXIST:
	    pass
	else:
	    raise Exception("Failed to create %s. (%s, %s)" %(CONFIG_DIR, errno, strerror))
	# Change to the config dir.
	os.chdir(CONFIG_DIR)
	conn = sqlite3.connect('www2sms.db')
	c = conn.cursor()
	# Create Database
	try:
	    c.execute('''create table login (id INTEGER PRIMARY KEY, provider TEXT, alias TEXT UNIQUE, login TEXT, password TEXT, last INTEGER)''')
	    #c.execute('''create table o2online (id INTEGER PRIMARY KEY, provider TEXT, alias TEXT UNIQUE)''')
	    conn.commit()
	except sqlite3.OperationalError:
	    pass
	c.close()
	conn.close()

# Connect to DB
mk_db()
conn = sqlite3.connect('www2sms.db')
c = conn.cursor()

if __name__ == "__main__":
	# Someone is launching this directly
	# Create the QApplication
	app = QtGui.QApplication(sys.argv)
	#The Status Form
	#Stat = FormPro()
	#The Account Form
	AccSet = FormAccount()
	#The O2 Advanced Form
	AdvSet = FormAdv()
	#The Main window
	ui = MainWindow()
	ui.show()
	# Enter the main loop
	app.exec_()
